from core.database import CursorFromPool
from core.eea.attainment import Attainment
from core.eea.exceedance_description import Exceedancedescription
from core.eea.exceeding_method import Exceedingmethod
from core.eea.network import Network
from core.eea.observation import Observation
from core.eea.station import Station
from core.eea.process import Process
from core.eea.observing_capabilities import ObservingCapabilities
from core.eea.sampling_point import SamplingPoint
from core.eea.responsible_reporter import ResponsibleReporter
from core.eea.sample import Sample
from core.eea.zone import Zone
from core.eea.pollutant import Pollutant
from core.eea.assessment_regime import Assessmentregime
from core.eea.assessment_method import Assessmentmethod
from core.eea.schema import Schema


class Dataflows:

    @staticmethod
    def get_xml(type: str, year: int, timezone: str, description: str):
        if type.upper() == "B":
            return Dataflows.get_dataflowB(year, timezone, description)
        if type.upper() == "C":
            return Dataflows.get_dataflowC(year, timezone, description)
        if type.upper() == "D":
            return Dataflows.get_dataflowD(year, timezone, description)
        if type.upper() == "E1A":
            return Dataflows.get_dataflowE1A(year, timezone, description)
        if type.upper() == "G":
            return Dataflows.get_dataflowG(year, timezone, description)
        return None

    ### DATAFLOWS ###

    @staticmethod
    def get_dataflowB(year: int, timezone: int, description: str):
        schema = Dataflows.get_schema()
        reporter = Dataflows.get_responsible_reporter(year, timezone, description)
        zones = Dataflows.get_zones()
        pollutants = Dataflows.get_pollutants()
        return schema.get_dataflow_b(reporter, zones, pollutants, timezone)

    @staticmethod
    def get_dataflowC(year: int, timezone: int, description: str):
        schema = Dataflows.get_schema()
        reporter = Dataflows.get_responsible_reporter(year, timezone, description)
        assessmentregimes = Dataflows.get_assessmentregimes()
        assessmentmethods = Dataflows.get_assessmentmethods()
        return schema.get_dataflow_c(reporter, assessmentregimes, assessmentmethods, timezone)

    @staticmethod
    def get_dataflowD(year: int, timezone: int, description: str):
        schema = Dataflows.get_schema()
        reporter = Dataflows.get_responsible_reporter(year, timezone, description)
        networks = Dataflows.get_networks()
        stations = Dataflows.get_stations()
        samples = Dataflows.get_samples()
        processes = Dataflows.get_processes()

        sampling_points = Dataflows.get_samplingpoints()
        for s in sampling_points:  # This is slow
            s.observing_capabilities = Dataflows.get_observing_capabilities(s.id)

        return schema.get_dataflow_d(reporter, networks, stations, sampling_points, samples, processes)

    @staticmethod
    def get_dataflowE1A(year: int, timezone: int, description: str):
        schema = Dataflows.get_schema()
        reporter = Dataflows.get_responsible_reporter(year, timezone, description)
        observations = Dataflows.get_observations_e1a(year)
        return schema.get_dataflow_e(reporter, observations)

    @staticmethod
    def get_dataflowE2A(last_request: str):
        schema = Dataflows.get_schema()
        observations = Dataflows.get_observations_e2a(last_request)
        if len(observations) == 0:
            raise Exception("No data available to create xml")
        reporter = Dataflows.get_responsible_reporter_simple()
        reporter.change = True
        reporter.begin = min([x.min_end_position for x in observations])
        reporter.end = max([x.max_end_position for x in observations])

        return schema.get_dataflow_e(reporter, observations)

    @staticmethod
    def get_dataflowG(year: int, timezone: int, description: str):
        schema = Dataflows.get_schema()
        reporter = Dataflows.get_responsible_reporter(year, timezone, description)
        attainments = Dataflows.get_attainments()
        exceedancedescriptions = Dataflows.get_exceedancedescriptions()
        exceedingmethods = Dataflows.get_exceedingmethods()
        return schema.get_dataflow_g(reporter, attainments, exceedancedescriptions, exceedingmethods, timezone)

    ### LOOKUPS ##
    @staticmethod
    def get_schema():
        with CursorFromPool() as cursor:
            cursor.execute("select * from settings")
            row = cursor.fetchone()
            return Dataflows.map_dict(row, Schema)

    @staticmethod
    def get_responsible_reporter(year: int, timezone: int, description: str):
        reporter = Dataflows.get_responsible_reporter_simple()
        reporter.change = True
        reporter.description = description if description else "XML generated by Raven"
        reporter.begin = str(year) + "-01-01T00:00:00+0" + str(timezone) + ":00"
        reporter.end = str(year+1) + "-01-01T00:00:00+0" + str(timezone) + ":00"
        return reporter

    @staticmethod
    def get_responsible_reporter_simple():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select *
                from responsible_authorities
                where is_responsible_reporter = true
              """)
            row = cursor.fetchone()
            reporter = Dataflows.map_dict(row, ResponsibleReporter)
            return reporter

    @staticmethod
    def get_zones():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT
                z."id",
                z."name",
                z.code,
                z.type,
                z.zone_type_uri ,
                z.population,
                z.population_year,
                z."year",
                z.area,
                ra."name" ra_name,
                ST_AsGML(3,z.geom,15,17,'gml', z."name") gml,
                ra.organisation ra_organisation,
                ra."locator" ra_locator,
                ra.postcode ra_postcode,
                ra.email ra_email,
                ra.address ra_address,
                ra.phone ra_phone,
                ra.website ra_website
            FROM
                zones z
                INNER JOIN responsible_authorities ra ON ra."id" = z.responsible_authority_id
            ORDER BY z.code
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Zone)

    @staticmethod
    def get_pollutants():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select 
                    ar.zoneid AS zone_id, 
                    ar.pollutant AS "uri", 
                    ar.protectiontarget AS target  
                from  
                    assessmentregimes ar
                    where ar.zoneid is not null
                group by 
                    zoneid,
                    pollutant, 
                    protectiontarget 
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Pollutant)

    @staticmethod
    def get_assessmentregimes():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT  a.id, 
                        a.name, 
                        a.zoneid, 
                        a.pollutant, 
                        o.uri as objecttype, 
                        r.uri as reportingmetric, 
                        p.uri as protectiontarget, 
                        t.uri as assessmentthresholdexceedance, 
                        a.thresholdclassificationyear,
                        a.thresholdclassificationreport
                
                FROM    assessmentregimes a, eea_objecttypes o, eea_reportingmetrics r, 
                        eea_protectiontargets p, eea_assessmentthresholdexceedances t

                WHERE a.include = true
                AND o.id = a.objecttype
                AND r.id = a.reportingmetric
                AND p.id = a.protectiontarget
                AND t.id = a.assessmentthresholdexceedance
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Assessmentregime)

    @staticmethod
    def get_assessmentmethods():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT  assessmentregime_id,  assessmenttype,
                        array_to_string(array_agg(assessmentlocal_id),',') AS assessmentlocal_id,
                        array_to_string(array_agg(assessmentmethodedescription),',') AS assessmentmethodedescription
                FROM public.assessmentdata               
                group by assessmentregime_id, assessmenttype
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Assessmentmethod)

    @staticmethod
    def get_networks():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select
                    n.*,
                    r.name rname, r.organisation rorganisation,
                    r.locator rlocator, r.postcode rpostcode,
                    r.email remail, r.address raddress,
                    r.phone rphone, r.website rwebsite
                from networks n, responsible_authorities r
                where n.responsible_authority_id = r.id 
                and (select count(*) from sampling_points s_sp, stations s_s where s_sp.station_id = s_s.id and s_s.network_id = n.id and s_sp.private = false) > 0
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Network)

    @staticmethod
    def get_stations():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT st.id,
                      st.name,
                      st.begin_position,
                      st.end_position,
                      st.network_id,
                      st.municipality,
                      st.eoi_code,
                      st.national_station_code,
                      st.media_monitored,
                      st.mobile,
                      st.measurement_regime,
                      st.area_classification,distance_junction,
                      st.traffic_volume,heavy_duty_fraction,
                      st.street_width,height_facades,
                      ST_X(st.geom) longitude,
                      ST_Y(st.geom) latitude,
                      ST_Z(st.geom) altitude,
                      ST_SRID(st.geom) epsg
                  FROM stations st
                  WHERE (select count(*) from sampling_points s_sp, stations s_s where s_sp.station_id = st.id and s_sp.private = false) > 0
                  GROUP BY
                      st.id,  
                      st.name,
                      st.begin_position,
                      st.end_position,
                      st.network_id,
                      st.municipality,
                      st.eoi_code,
                      st.national_station_code,
                      st.media_monitored,
                      st.mobile,
                      st.measurement_regime,
                      st.area_classification,distance_junction,
                      st.traffic_volume,heavy_duty_fraction,
                      st.street_width,height_facades,
                      ST_X(st.geom),
                      ST_Y(st.geom),
                      ST_Z(st.geom),
                      ST_SRID(st.geom)
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Station)

    @staticmethod
    def get_processes():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select p.*,
                    r.name rname, r.organisation rorganisation,
                    r.locator rlocator, r.postcode rpostcode,
                    r.email remail, r.address raddress,
                    r.phone rphone, r.website rwebsite
                from processes p, responsible_authorities r
                where p.responsible_authority_id = r.id
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Process)

    @staticmethod
    def get_samplingpoints():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select
                    sp.*,
                    ST_X(st.geom) longitude,ST_Y(st.geom) latitude, ST_SRID(st.geom) epsg,
                    st.network_id
                from sampling_points sp, stations st
                where sp.station_id = st.id
                and sp.private = false 
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, SamplingPoint)

    @staticmethod
    def get_observing_capabilities(sampling_point_id: str):
        with CursorFromPool() as cursor:
            cursor.execute("""
                select oc.*, sp.pollutant
                from observing_capabilities oc, sampling_points sp
                where oc.sampling_point_id = sp.id
                and sp.id =   %(sampling_point_id)s
                and sp.private = false
              """, {"sampling_point_id": sampling_point_id})
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, ObservingCapabilities)

    @staticmethod
    def get_samples():
        with CursorFromPool() as cursor:
            cursor.execute("""
                select
                    sa.id,
                    sa.inlet_height,
                    sa.building_distance,
                    sa.kerb_distance,
                    ST_X(st.geom) longitude,ST_Y(st.geom) latitude, ST_SRID(st.geom) epsg
                from sampling_points sp, stations st, samples sa, observing_capabilities oc
                where sp.station_id = st.id            
                and sp.id = oc.sampling_point_id
                and oc.sample_id = sa.id
                and sp.private = false
                group by (sa.id, sa.inlet_height,sa.building_distance,sa.kerb_distance,longitude, latitude, epsg)
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Sample)

    @staticmethod
    def get_observations_e1a(year: int):
        with CursorFromPool() as cursor:
            cursor.execute("""
                select max(c.id) id, c.process_id,s.assessment_type,s.pollutant, c.sample_id,
                o.sampling_point_id, s.concentration, s.timestep,
                count(o.sampling_point_id) cnt, min(o.begin_position) min_begin_position, max(o.begin_position) max_begin_position,min(o.end_position) min_end_position, max(o.end_position) max_end_position,
                string_agg(o.begin_position || ',' || o.end_position || ',' || o.verification_flag || ',' || o.validation_flag || ',' || o.value,'@@' order by o.begin_position) vals
                from observations o, sampling_points s, observing_capabilities c
                where o.sampling_point_id = s.id
                and c.sampling_point_id = s.id
                and (o.end_position >= c.begin_position)
                and o.end_position < COALESCE(c.end_position, '9999-01-01T00:00:00+01:00' )
                and EXTRACT(YEAR FROM to_date(o.begin_position,'yyyy-mm-dd')) = %(year)s
                and s.private = false
                group by c.begin_position, c.process_id,s.assessment_type,s.pollutant, c.sample_id, o.sampling_point_id, s.concentration, s.timestep
              """, {"year": year})
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Observation)

    @staticmethod
    def get_observations_e2a(last_request: str):
        with CursorFromPool() as cursor:
            cursor.execute("""
                select max(c.id) id, c.process_id,s.assessment_type,s.pollutant, c.sample_id,
                o.sampling_point_id, s.concentration, s.timestep,
                count(o.sampling_point_id) cnt, min(o.begin_position) min_begin_position, max(o.begin_position) max_begin_position,min(o.end_position) min_end_position, max(o.end_position) max_end_position,
                string_agg(o.begin_position || ',' || o.end_position || ',' || o.verification_flag || ',' || o.validation_flag || ',' || o.value,'@@' order by o.begin_position) vals
                from observations o, sampling_points s, observing_capabilities c
                where o.sampling_point_id = s.id
                and c.sampling_point_id = s.id
                and (o.end_position >= c.begin_position)
                and o.end_position < COALESCE(c.end_position, '9999-01-01T00:00:00+01:00' )
                and o.touched > %(last_request)s
                and s.private = false
                group by c.begin_position, c.process_id,s.assessment_type,s.pollutant, c.sample_id, o.sampling_point_id, s.concentration, s.timestep
              """, {"last_request": last_request})
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Observation)

    @staticmethod
    def get_attainments():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT 	a.id, a.name, a.assessmentregime_id, a.comment, 
                        ar.zoneid, 	ar.pollutant, o.uri as objecttype, r.uri as reportingmetric, p.uri as protectiontarget
                FROM 	attainments a, assessmentregimes ar, eea_objecttypes o,	eea_reportingmetrics r,	eea_protectiontargets p
                WHERE 	ar.include = true
                        AND ar.id = a.assessmentregime_id
                        AND o.id = ar.objecttype
                        AND r.id = ar.reportingmetric
                        AND p.id = ar.protectiontarget
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Attainment)

    @staticmethod
    def get_exceedancedescriptions():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT
                  ex.id,
                  ex.attainment_id,
                  ex.exceedances, 
                  ex.excedance_type, 
                  ex.max_value, 
                  et.adjustment_type,
                            es.adjustment_source,
                  ex.surface_area,
                  ex.exposed_population,
                  ex.population_reference_year, 
                  ex.vegetation_area, 
                  ex.area_classification, 
                  ex.exceedancedescription_element,
                  er.exceedance_reason, 
                  ex.other_exceedance_reason FROM
                      (
                    SELECT 
                                e.id,
                                e.attainment_id,
                                e.exceedances, 
                                et.name as excedance_type, 
                                e.max_value, 
                                e.adjustment_type,
                                e.adjustment_source,
                                e.surface_area,
                                e.exposed_population,
                                e.population_reference_year, 
                                e.vegetation_area, 
                                e.area_classification, 
                                e.exceedance_reason,
                                e.other_exceedance_reason, 
                                ex.id as exceedancedescription_element_id,
                      ex.name as exceedancedescription_element
                    FROM exceedancedescriptions e, attainments at, assessmentregimes a, eea_exceedancetype et, eea_exceedancedescription ex
                    WHERE a.include = true
                    AND a.id = at.assessmentregime_id
                    AND at.id = e.attainment_id
                    AND et.id = e.excedance_type
                    AND ex.id = e.exceedancedescription_element
                ) as ex
                  LEFT OUTER JOIN	(SELECT id, uri as exceedance_reason FROM eea_exceedancereason) AS er ON ex.exceedance_reason = er.id
                  LEFT OUTER JOIN	(SELECT id, uri as adjustment_type FROM eea_adjustmenttypes) AS et ON ex.adjustment_type = et.id
                  LEFT OUTER JOIN	(SELECT id, uri as adjustment_source FROM eea_adjustmentsourcetype) AS es ON ex.adjustment_source = es.id
                  ORDER BY ex.attainment_id, exceedancedescription_element_id
              """)
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Exceedancedescription)

    @staticmethod
    def get_exceedingmethods():
        with CursorFromPool() as cursor:
            cursor.execute("""
                SELECT ed.id as exceedancedescription_id, ass.assessmentlocal_id
                FROM exceedancedescriptions ed,
                  attainments at,
                  assessmentregimes ar,
                  assessmentdata ass
              WHERE 1=1
                AND ar.include = true
                AND ed.attainment_id = at.id
                AND at.assessmentregime_id = ar.id
                AND ass.assessmentregime_id = ar.id
              """
                           )
            rows = cursor.fetchall()
            return Dataflows.map_list_of_dict(rows, Exceedingmethod)

    ### HELPERS ###
    @staticmethod
    def map_dict(dict, obj):
        o = obj()
        if dict is None:
            return o

        for p in dict:
            if hasattr(o, p):
                setattr(o, p, dict[p])
        return o

    @staticmethod
    def map_list_of_dict(lst, obj):
        ret = list()
        if len(lst) == 0:
            return ret

        for l in lst:
            ret.append(Dataflows.map_dict(l, obj))
        return ret
